@@ -15,7 +15,7 @@ class RichFormatter(base.ListFormatter, base.SingleFormatter):
         )
         for row in data:
             table.add_row(*[str(cell) for cell in row])
-        Console().print(table)
+        Console(emoji=False, markup=False).print(table)
 
     def emit_one(self, column_names, data, stdout, parsed_args):
         table = Table(
@@ -26,4 +26,4 @@ class RichFormatter(base.ListFormatter, base.SingleFormatter):
         )
         for key, value in list(zip(column_names, data)):
             table.add_row(key, str(value))
-        Console().print(table)
+        Console(emoji=False, markup=False).print(table)
