@@ -2,16 +2,11 @@
     DigiCloud load balancer Service.
 """
 import ipaddress
-
+from ..utils import is_tty
 from .base import Lister, ShowOne, Command
 from .. import schemas
 from ..error_handlers import CLIError
-
-
-def get_load_balancer_details(session, load_balancer):
-    uri = '/load-balancers/%s' % load_balancer
-    load_balancer = session.get(uri)
-    return load_balancer
+from rich.prompt import Confirm
 
 
 ################################################
@@ -55,12 +50,39 @@ class DeleteLoadBalancer(Command):
             metavar='<load-balancer>',
             help='Load Balancer name or ID',
         )
+        parser.add_argument(
+            '--i-am-sure',
+            help='Use this switch to bypass confirmation',
+            default=None,
+            action='store_true'
+        )
+
         return parser
 
     def take_action(self, parsed_args):
+        self.confirm_deletion(parsed_args)
         uri = '/load-balancers/%s' % parsed_args.load_balancer
         self.app.session.delete(uri)
 
+    def confirm_deletion(self, parsed_args):
+        if parsed_args.i_am_sure:
+            return True
+        if is_tty():
+            user_response = Confirm.ask(
+                "You're about to delete loadbalancer [red bold]{}[/red bold]. "
+                "Are you sure?".format(
+                    parsed_args.load_balancer
+                ), default=False
+            )
+            if user_response:
+                return True
+            self.app.stdout.write("Operation cancelled by user\n")
+        else:
+            self.app.stderr.write(
+                "Unable to perform 'loadbalancer delete' operation in non-interactive mode,"
+                " without '--i-am-sure' switch\n")
+            return False
+
 
 class UpdateLoadBalancer(ShowOne):
     """Update load balancer."""
@@ -83,99 +105,19 @@ class UpdateLoadBalancer(ShowOne):
             metavar='<description>',
             help='Load balancer description.',
         )
-        parser.add_argument(
-            '--delay',
-            metavar='<delay>',
-            help='Health Check delay.',
-            type=int,
-        )
-        parser.add_argument(
-            '--timeout',
-            metavar='<timeout>',
-            help='Health Check timeout.',
-            type=int,
-        )
-        parser.add_argument(
-            '--max-retries',
-            metavar='<max-retries>',
-            help='Health Check max retries.',
-            type=int,
-        )
-        parser.add_argument(
-            '--path',
-            metavar='<path>',
-            help='Health Check path',
-        )
-        parser.add_argument(
-            '--http-version',
-            metavar='<http-version>',
-            help='Health Check http version',
-            type=float,
-            choices=[1.0, 1.1],
-        )
-        parser.add_argument(
-            '--http-method',
-            metavar='<http-method>',
-            help='Health Check http method',
-            choices=['GET', 'POST'],
-        )
-        parser.add_argument(
-            '--algorithm',
-            metavar='<algorithm>',
-            help='Backend algorithm',
-            choices=['round_robin', 'source_ip'],
-        )
         return parser
 
     def get_data(self, parsed_args):
         uri = '/load-balancers/%s' % parsed_args.load_balancer
         payload = {}
-
-        health_check_payload = {}
-        if parsed_args.delay:
-            health_check_payload['delay'] = parsed_args.delay
-        if parsed_args.timeout:
-            health_check_payload['timeout'] = parsed_args.timeout
-        if parsed_args.max_retries:
-            health_check_payload['max_retries'] = parsed_args.max_retries
-        if parsed_args.path:
-            health_check_payload['path'] = parsed_args.path
-        if parsed_args.http_version:
-            health_check_payload['http_version'] = parsed_args.http_version
-        if parsed_args.http_method:
-            health_check_payload['http_method'] = parsed_args.http_method
-
         if parsed_args.name:
             payload['name'] = parsed_args.name
         if parsed_args.description:
             payload['description'] = parsed_args.description
 
-        if health_check_payload:
-            payload["application"] = {
-                "backends": [{
-                    "health_check": health_check_payload
-                }]
-            }
-
-        if parsed_args.algorithm:
-            if payload.get("application"):
-                payload["application"]["backends"][0]["algorithm"] = parsed_args.algorithm
-            else:
-                payload["application"] = {
-                    "backends": [{
-                        "algorithm": parsed_args.algorithm
-                    }]
-                }
-
         if not payload:
             raise CLIError([dict(
-                msg="At least one of --delay or"
-                    " --timeout or"
-                    " --max-retries or"
-                    " --path or"
-                    " --http-version or"
-                    " --http-method or"
-                    " --algorithm or"
+                msg="At least one of "
                     " --name or"
                     " --description is necessary"
             )])
@@ -190,10 +132,9 @@ class CreateLoadBalancer(ShowOne):
     def get_parser(self, prog_name):
         parser = super().get_parser(prog_name)
         parser.add_argument(
-            '--name',
+            'name',
             metavar='<Name>',
             help='New name for load balancer.',
-            required=True,
         )
         parser.add_argument(
             '--description',
@@ -213,11 +154,254 @@ class CreateLoadBalancer(ShowOne):
 
 
 ################################################
-# Application                                  #
+# Backend                                      #
 ################################################
-class CreateApplication(ShowOne):
-    """Create Application"""
-    schema = schemas.ApplicationDetail()
+
+
+class CreateBackend(ShowOne):
+    """Create Load balancer Backend.
+
+    Each backend may contain multiple backend members.
+    """
+    schema = schemas.BackendDetail()
+
+    def get_parser(self, prog_name):
+        parser = super().get_parser(prog_name)
+        parser.add_argument(
+            '--algorithm',
+            metavar='<algorithm>',
+            help='Backend algorithm one of "round_robin" or "source_ip"',
+            choices=['round_robin', 'source_ip'],
+            required=True
+        )
+        parser.add_argument(
+            'load_balancer',
+            metavar='<load-balancer>',
+            help='Load Balancer name or ID',
+        )
+        return parser
+
+    def get_data(self, parsed_args):
+        payload = {
+            'algorithm': parsed_args.algorithm,
+        }
+        uri = '/load-balancers/%s/backends' % parsed_args.load_balancer
+        backend = self.app.session.post(uri, payload)
+        return backend
+
+
+class DeleteBackend(Command):
+    """Delete a LoadBalancer backend."""
+    def get_parser(self, prog_name):
+        parser = super().get_parser(prog_name)
+        parser.add_argument(
+            'load_balancer',
+            metavar='<load-balancer>',
+            help='Load Balancer name or ID',
+        )
+        parser.add_argument(
+            '--backend-id',
+            metavar='<backend_id>',
+            required=True,
+            help='The backend identifier to be deleted.',
+        )
+        parser.add_argument(
+            '--i-am-sure',
+            help='Use this switch to bypass confirmation',
+            default=None,
+            action='store_true'
+        )
+
+        return parser
+
+    def take_action(self, parsed_args):
+        self.confirm_deletion(parsed_args)
+        uri = '/load-balancers/%s/backends/%s' % (
+            parsed_args.load_balancer,
+            parsed_args.backend_id,
+        )
+        self.app.session.delete(uri)
+
+    def confirm_deletion(self, parsed_args):
+        if parsed_args.i_am_sure:
+            return True
+        if is_tty():
+            user_response = Confirm.ask(
+                "You're about to delete backend "
+                " [red bold]{}[/red bold] "
+                "including the members and the health check. "
+                "Are you sure?".format(
+                    parsed_args.backend_id
+                ), default=False
+            )
+            if user_response:
+                return True
+            self.app.stdout.write("Operation cancelled by user\n")
+        else:
+            self.app.stderr.write(
+                "Unable to perform 'backend delete' operation in non-interactive mode,"
+                " without '--i-am-sure' switch\n")
+            return False
+
+
+class ListBackend(Lister):
+    """Lists a load balancer backends."""
+    schema = schemas.BackendDetail(many=True)
+
+    def get_parser(self, prog_name):
+        parser = super().get_parser(prog_name)
+        parser.add_argument(
+            'load_balancer',
+            metavar='<load-balancer>',
+            help='Load Balancer name or ID',
+        )
+        return parser
+
+    def get_data(self, parsed_args):
+        uri = '/load-balancers/%s/backends' % parsed_args.load_balancer
+        backends = self.app.session.get(uri)
+        return backends
+
+
+################################################
+# Frontend                                     #
+################################################
+
+
+class CreateFrontend(ShowOne):
+    """Create Load balancer Frontend."""
+    schema = schemas.FrontendDetail()
+
+    def get_parser(self, prog_name):
+        parser = super().get_parser(prog_name)
+        parser.add_argument(
+            '--port',
+            metavar='<port>',
+            help='The TCP port which frontend listens to.',
+            type=int,
+            required=True,
+        )
+        parser.add_argument(
+            '--timeout-client-data',
+            metavar='<timeout_client_data>',
+            help='Client inactivity timeout in milliseconds.',
+            type=int,
+            required=False,
+        )
+        parser.add_argument(
+            '--timeout-tcp-inspect',
+            metavar='<timeout_tcp_inspect>',
+            help='Time, in milliseconds, '
+                 'to wait for additional TCP packets for content inspection.',
+            type=int,
+            required=False,
+        )
+        parser.add_argument(
+            '--timeout-member-connect',
+            metavar='<timeout_member_connect>',
+            help='Backend member connection timeout in milliseconds.',
+            type=int,
+            required=False,
+        )
+        parser.add_argument(
+            '--timeout_member_data',
+            metavar='<timeout_member_data>',
+            help='Backend member inactivity timeout in milliseconds.',
+            type=int,
+            required=False,
+        )
+        parser.add_argument(
+            '--connection-limit',
+            metavar='<connection_limit>',
+            help='The maximum number of connections permitted for this Frontend.'
+                 'Value "-1" represents infinite connections.',
+            type=int,
+            required=True,
+        )
+        parser.add_argument(
+            'load_balancer',
+            metavar='<load-balancer>',
+            help='Load Balancer name or ID',
+        )
+        parser.add_argument(
+            '--default-backend',
+            metavar='<default_backend>',
+            required=True,
+            help='default backend which the frontend will forward requests to.',
+        )
+        return parser
+
+    def get_data(self, parsed_args):
+        payload = {
+            'port': parsed_args.port,
+            'default_backend_id': parsed_args.default_backend,
+            "connection_limit": parsed_args.connection_limit,
+        }
+        if parsed_args.timeout_client_data:
+            payload["timeout_client_data"] = parsed_args.timeout_client_data
+        if parsed_args.timeout_tcp_inspect:
+            payload["timeout_tcp_inspect"] = parsed_args.timeout_tcp_inspect
+        if parsed_args.timeout_member_connect:
+            payload["timeout_member_connect"] = parsed_args.timeout_member_connect
+        if parsed_args.timeout_member_data:
+            payload["timeout_member_data"] = parsed_args.timeout_member_data
+
+        uri = '/load-balancers/%s/frontends' % parsed_args.load_balancer
+        frontend = self.app.session.post(uri, payload)
+        return frontend
+
+
+class DeleteFrontend(Command):
+    """Delete a LoadBalancer frontend."""
+    def get_parser(self, prog_name):
+        parser = super().get_parser(prog_name)
+        parser.add_argument(
+            'load_balancer',
+            metavar='<load-balancer>',
+            help='Load Balancer name or ID',
+        )
+        parser.add_argument(
+            '--frontend-id',
+            metavar='<frontend_id>',
+            required=True,
+            help='The frontend identifier to be deleted.',
+        )
+        return parser
+
+    def take_action(self, parsed_args):
+        uri = '/load-balancers/%s/frontends/%s' % (
+            parsed_args.load_balancer,
+            parsed_args.frontend_id,
+        )
+        self.app.session.delete(uri)
+
+
+class ListFrontend(Lister):
+    """Lists a load balancer frontends."""
+    schema = schemas.FrontendDetail(many=True)
+
+    def get_parser(self, prog_name):
+        parser = super().get_parser(prog_name)
+        parser.add_argument(
+            'load_balancer',
+            metavar='<load-balancer>',
+            help='Load Balancer name or ID',
+        )
+        return parser
+
+    def get_data(self, parsed_args):
+        uri = '/load-balancers/%s/frontends' % parsed_args.load_balancer
+        frontends = self.app.session.get(uri)
+        return frontends
+
+################################################
+# Health Check                                 #
+################################################
+
+
+class ListHealthCheck(Lister):
+    """Lists backend Health check."""
+    schema = schemas.HealthCheckDetail(many=True)
 
     def get_parser(self, prog_name):
         parser = super().get_parser(prog_name)
@@ -227,57 +411,22 @@ class CreateApplication(ShowOne):
             help='Load Balancer name or ID',
         )
         parser.add_argument(
-            '--port',
-            metavar='<port>',
-            help='Frontend port.',
-            type=int,
-            required=True,
-        )
-        parser.add_argument(
-            '--algorithm',
-            metavar='<algorithm>',
-            help='Backend algorithm',
-            choices=['round_robin', 'source_ip'],
+            '--backend-id',
+            metavar='<backend_id>',
             required=True,
+            help='The backend identifier.',
         )
         return parser
 
-    def _check_arg_validity(self, parsed_args):
-        rules = [
-            (
-                parsed_args.port < 1 or parsed_args.port > 65535,
-                "port should be in range 1 - 65535",
-            ),
-        ]
-        errors = []
-        for is_invalid, err_msg in rules:
-            if is_invalid:
-                errors.append(dict(msg=err_msg))
-        if errors:
-            raise CLIError(errors)
-
     def get_data(self, parsed_args):
-        self._check_arg_validity(parsed_args)
-        payload = {
-            "backends": [
-                {
-                    'algorithm': parsed_args.algorithm,
-                },
-            ],
-            "frontends": [
-                {
-                    'port': parsed_args.port,
-                },
-            ],
-        }
-        uri = '/load-balancers/%s/applications' % parsed_args.load_balancer
-        application = self.app.session.post(uri, payload)
-        return application
+        uri = '/load-balancers/%s/backends/%s/health-checks' % (
+            parsed_args.load_balancer,
+            parsed_args.backend_id,
+        )
+        health_checks = self.app.session.get(uri)
+        return health_checks
 
 
-################################################
-# Health Check                                 #
-################################################
 class DeleteHealthCheck(Command):
     """Delete Health Check."""
 
@@ -288,15 +437,25 @@ class DeleteHealthCheck(Command):
             metavar='<load-balancer>',
             help='Load Balancer name or ID',
         )
+        parser.add_argument(
+            '--backend-id',
+            metavar='<backend_id>',
+            required=True,
+            help='The backend identifier.',
+        )
+        parser.add_argument(
+            '--health-check-id',
+            metavar='<health_check_id>',
+            required=True,
+            help='The health check identifier to be deleted.',
+        )
         return parser
 
     def take_action(self, parsed_args):
-        load_balancer = get_load_balancer_details(self.app.session, parsed_args.load_balancer)
-        uri = '/load-balancers/%s/applications/%s/backends/%s/health-checks/%s' % (
+        uri = '/load-balancers/%s/backends/%s/health-checks/%s' % (
             parsed_args.load_balancer,
-            load_balancer['application']['id'],
-            load_balancer['application']['backends'][0]["id"],
-            load_balancer['application']['backends'][0]['health_check']['id'],
+            parsed_args.backend_id,
+            parsed_args.health_check_id,
         )
         self.app.session.delete(uri)
 
@@ -312,6 +471,18 @@ class CreateHealthCheck(ShowOne):
             metavar='<load-balancer>',
             help='Load Balancer name or ID',
         )
+        parser.add_argument(
+            '--backend-id',
+            metavar='<backend_id>',
+            required=True,
+            help='The backend identifier.',
+        )
+        parser.add_argument(
+            '--name',
+            metavar='<Name>',
+            help='Human readable name.',
+            required=True,
+        )
         parser.add_argument(
             '--delay',
             metavar='<delay>',
@@ -334,22 +505,13 @@ class CreateHealthCheck(ShowOne):
             required=True,
         )
         parser.add_argument(
-            '--path',
-            metavar='<path>',
-            help='Health Check path',
-        )
-        parser.add_argument(
-            '--http-version',
-            metavar='<http-version>',
-            help='Health Check http version',
-            type=float,
-            choices=[1.0, 1.1],
-        )
-        parser.add_argument(
-            '--http-method',
-            metavar='<http-method>',
-            help='Health Check http method',
-            choices=['GET', 'POST'],
+            '--max-retries-down',
+            metavar='<max-retries-down>',
+            help='The number of allowed check failures before '
+                 'changing the operating status of the member to ERROR. '
+                 'A valid value is from 1 to 10.',
+            type=int,
+            required=True,
         )
         return parser
 
@@ -369,19 +531,16 @@ class CreateHealthCheck(ShowOne):
 
     def get_data(self, parsed_args):
         self._check_arg_validity(parsed_args)
-        load_balancer = get_load_balancer_details(self.app.session, parsed_args.load_balancer)
-        uri = '/load-balancers/%s/applications/%s/backends/%s/health-checks' % (
+        uri = '/load-balancers/%s/backends/%s/health-checks' % (
             parsed_args.load_balancer,
-            load_balancer['application']['id'],
-            load_balancer['application']['backends'][0]["id"],
+            parsed_args.backend_id,
         )
         payload = {
+            'name': parsed_args.name,
             'delay': parsed_args.delay,
             'timeout': parsed_args.timeout,
             'max_retries': parsed_args.max_retries,
-            'path': parsed_args.path,
-            'http_version': parsed_args.http_version,
-            'http_method': parsed_args.http_method,
+            'max_retries_down': parsed_args.max_retries_down,
         }
         health_check = self.app.session.post(uri, payload)
         return health_check
@@ -390,6 +549,35 @@ class CreateHealthCheck(ShowOne):
 ################################################
 # Backend Member                               #
 ################################################
+
+class ListBackendMember(Lister):
+    """Lists backend members."""
+    schema = schemas.BackendMemberDetail(many=True)
+
+    def get_parser(self, prog_name):
+        parser = super().get_parser(prog_name)
+        parser.add_argument(
+            'load_balancer',
+            metavar='<load-balancer>',
+            help='Load Balancer name or ID',
+        )
+        parser.add_argument(
+            '--backend-id',
+            metavar='<backend_id>',
+            required=True,
+            help='The backend identifier.',
+        )
+        return parser
+
+    def get_data(self, parsed_args):
+        uri = '/load-balancers/%s/backends/%s/members' % (
+            parsed_args.load_balancer,
+            parsed_args.backend_id,
+        )
+        members = self.app.session.get(uri)
+        return members
+
+
 class DeleteBackendMember(Command):
     """Delete backend member."""
 
@@ -401,22 +589,54 @@ class DeleteBackendMember(Command):
             help='Load Balancer name or ID',
         )
         parser.add_argument(
-            'member',
-            metavar='<member>',
-            help='Member ID',
+            '--backend-id',
+            metavar='<backend_id>',
+            required=True,
+            help='The backend identifier.',
+        )
+        parser.add_argument(
+            '--member-id',
+            metavar='<member_id>',
+            help='The member identifier to be deleted.',
+            required=True
+        )
+        parser.add_argument(
+            '--i-am-sure',
+            help='Use this switch to bypass confirmation',
+            default=None,
+            action='store_true'
         )
         return parser
 
     def take_action(self, parsed_args):
-        load_balancer = get_load_balancer_details(self.app.session, parsed_args.load_balancer)
-        uri = '/load-balancers/%s/applications/%s/backends/%s/members/%s' % (
+        self.confirm_deletion(parsed_args)
+        uri = '/load-balancers/%s/backends/%s/members/%s' % (
             parsed_args.load_balancer,
-            load_balancer['application']['id'],
-            load_balancer['application']['backends'][0]["id"],
-            parsed_args.member,
+            parsed_args.backend_id,
+            parsed_args.member_id,
         )
         self.app.session.delete(uri)
 
+    def confirm_deletion(self, parsed_args):
+        if parsed_args.i_am_sure:
+            return True
+        if is_tty():
+            user_response = Confirm.ask(
+                "You're about to delete backend member"
+                " [red bold]{}[/red bold] . "
+                "Are you sure?".format(
+                    parsed_args.member_id
+                ), default=False
+            )
+            if user_response:
+                return True
+            self.app.stdout.write("Operation cancelled by user\n")
+        else:
+            self.app.stderr.write(
+                "Unable to perform 'backend member delete' operation in non-interactive mode,"
+                " without '--i-am-sure' switch\n")
+            return False
+
 
 class AddBackendMember(Lister):
     """Add Backend member"""
@@ -430,10 +650,10 @@ class AddBackendMember(Lister):
             help='Load Balancer name or ID',
         )
         parser.add_argument(
-            '--resource-id',
-            metavar='<resource-id>',
-            help='Backend resource id.',
+            '--backend-id',
+            metavar='<backend_id>',
             required=True,
+            help='The backend identifier.',
         )
         parser.add_argument(
             '--ip-address',
@@ -477,14 +697,11 @@ class AddBackendMember(Lister):
 
     def get_data(self, parsed_args):
         self._check_arg_validity(parsed_args)
-        load_balancer = get_load_balancer_details(self.app.session, parsed_args.load_balancer)
-        uri = '/load-balancers/%s/applications/%s/backends/%s/members' % (
+        uri = '/load-balancers/%s/backends/%s/members' % (
             parsed_args.load_balancer,
-            load_balancer['application']['id'],
-            load_balancer['application']['backends'][0]["id"],
+            parsed_args.backend_id,
         )
         payload = [{
-            'resource_id': parsed_args.resource_id,
             'ip_address': parsed_args.ip_address,
             'port': parsed_args.port,
         }]
