@@ -1,19 +1,17 @@
-from cliff.command import Command
-from cliff.lister import Lister
-from cliff.show import ShowOne
-
-from digicloud.utils import tabulate
+from digicloud.commands.base import Lister, ShowOne
+from digicloud.error_handlers import CLIError
 
 
 class ListRegions(Lister):
     """List available regions"""
-    def take_action(self, parsed_args):
-        data = self.app.session.get('/regions')
-        return tabulate(data)
+
+    def get_data(self, parsed_args):
+        return self.app.session.get('/regions')
 
 
-class SelectRegion(Command):
+class SelectRegion(ShowOne):
     """Select specific region to setup"""
+
     def get_parser(self, prog_name):
         parser = super(SelectRegion, self).get_parser(prog_name)
 
@@ -25,19 +23,28 @@ class SelectRegion(Command):
 
         return parser
 
-    def take_action(self, parsed_args):
+    def get_data(self, parsed_args):
+        regions = {region['name'] for region in self.app.session.get('/regions')}
+        if parsed_args.region_name not in regions:
+            raise CLIError([dict(
+                msg="We have no such a region in Digicloud",
+                hint='Check our current regions via '
+                     '[blue bold]digicloud region list[/blue bold]'
+            )])
         self.app.config['AUTH_HEADERS'].update({
             'Digicloud-Region': parsed_args.region_name,
         })
+        return {'Region': parsed_args.region_name}
 
 
 class CurrentRegion(ShowOne):
     """Show current selected region"""
-    def take_action(self, parsed_args):
+
+    def get_data(self, parsed_args):
         header = self.app.config.get('AUTH_HEADERS')
         if header is not None:
             selected_region = header.get('Digicloud-Region', 'No region selected')
         else:
             selected_region = 'No region selected'
 
-        return ('Current region', ), (selected_region, )
+        return {'Region': selected_region}
