@@ -585,12 +585,17 @@ class SRVRecordDetailsSchema(RecordListSchema):
 
 
 class HealthCheckDetail(Schema):
+    id = fields.Str()
+    name = fields.Str()
+    openstack_id = fields.Str()
     delay = fields.Int()
-    timeout = fields.Int()
     max_retries = fields.Int()
-    path = fields.Str()
-    http_version = fields.Float()
-    http_method = fields.Str()
+    max_retries_down = fields.Int()
+    timeout = fields.Int()
+    type = fields.Str()
+    operating_status = fields.Str()
+    created_at = APIDatetime()
+    updated_at = APIDatetime()
 
     class Meta:
         ordered = True
@@ -598,63 +603,45 @@ class HealthCheckDetail(Schema):
 
 class BackendMemberDetail(Schema):
     id = fields.Str()
-    resource_id = fields.Str()
     ip_address = fields.Str()
-    port = fields.Int()
+    port = fields.Str()
+    operating_status = fields.Str()
+    created_at = APIDatetime()
+    updated_at = APIDatetime()
 
     class Meta:
         ordered = True
 
 
 class BackendDetail(Schema):
+    id = fields.Str()
     algorithm = fields.Str()
-    health_check = fields.Nested(HealthCheckDetail)
-    members = fields.List(fields.Nested(BackendMemberDetail))
+    protocol = fields.Str()
+    health_checks = fields.Function(lambda lb: [hc["id"] for hc in lb["health_checks"]])
+    members = fields.Function(lambda lb: [member["id"] for member in lb["members"]])
+    updated_at = APIDatetime()
+    created_at = APIDatetime()
 
     class Meta:
         ordered = True
 
 
 class FrontendDetail(Schema):
-    port = fields.Int()
-
-    class Meta:
-        ordered = True
-
-
-class ApplicationDetail(Schema):
     id = fields.Str()
-    backends = fields.List(fields.Nested(BackendDetail))
-    frontends = fields.List(fields.Nested(FrontendDetail))
+    port = fields.Str()
+    protocol = fields.Str()
+    timeout_client_data = fields.Int()
+    timeout_tcp_inspect = fields.Int()
+    timeout_member_connect = fields.Int()
+    timeout_member_data = fields.Int()
+    connection_limit = fields.Int()
+    updated_at = APIDatetime()
+    created_at = APIDatetime()
+    default_backend_id = fields.Str()
 
     class Meta:
         ordered = True
 
-    @post_dump
-    def handle_nested_data(self, data, **kwargs):
-        if data.get("frontends"):
-            handle_nested_data_keys(
-                data, FrontendDetail().dump(data["frontends"][0]), "frontend"
-            )
-        data.pop("frontends")
-        if data.get("backends"):
-            health_check = data["backends"][0].pop("health_check")
-            members = data["backends"][0].pop("members")
-            handle_nested_data_keys(data, BackendDetail().dump(data["backends"][0]), "backend")
-            if health_check:
-                handle_nested_data_keys(
-                    data, HealthCheckDetail().dump(health_check),
-                    "health_check",
-                )
-            if members:
-                handle_nested_data_keys(
-                    data,
-                    BackendMemberDetail(many=True).dump(members),
-                    "member",
-                )
-        data.pop("backends")
-        return data
-
 
 class LoadBalancerList(Schema):
     name = fields.Str()
@@ -672,7 +659,8 @@ class LoadBalancerDetail(Schema):
     name = fields.Str()
     description = fields.Str()
     status = fields.Str()
-    application = fields.Nested(ApplicationDetail)
+    backends = fields.Function(lambda lb: [backend["id"] for backend in lb["backends"]])
+    frontends = fields.Function(lambda lb: [frontend["id"] for frontend in lb["frontends"]])
     ip_address = fields.Str()
     created_at = APIDatetime()
     updated_at = APIDatetime()
@@ -680,17 +668,6 @@ class LoadBalancerDetail(Schema):
     class Meta:
         ordered = True
 
-    @post_dump
-    def handle_nested_data(self, data, **kwargs):
-        if data.get("application"):
-            if data["application"].get("id"):
-                data["application"].pop("id")
-                for key, value in data["application"].items():
-                    data[key] = value
-
-        data.pop("application")
-        return data
-
 
 def handle_nested_data_keys(data, nested_data, nested_key):
     if isinstance(nested_data, dict):
